﻿namespace ProjectSeaplane;
/// <summary>
/// Отрисовка и перемещение
/// </summary>
public class DrawingSeaplane
{
    /// <summary>
	/// Класс-сущность
	/// </summary>
	public EntitySeaplane? EntitySeaplane { get; private set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    private int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    private int? _startPosY;

    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningSeaplaneWidth = 110;

    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningSeaplaneHeight = 60;

    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="wingsOpened">Признак cворачивания крыла(0 - свернуто, 1 - развернуто)</param>
    /// <param name="screwEngine">Признак типа двигателя (1 - винтовой, 0 - реативный)</param>
    public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool wingsOpened, bool screwEngine)
    {
        EntitySeaplane = new EntitySeaplane();
        EntitySeaplane.Init(speed, weight, bodyColor, additionalColor, wingsOpened, screwEngine);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена



        _pictureWidth = width;
        _pictureHeight = height;

        if (_drawningSeaplaneWidth < width && _drawningSeaplaneHeight < height) {
            return true;
        }
        else { return false; }
        
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        _startPosX = x;
        _startPosY = y;

        if (_drawningSeaplaneWidth - _startPosX < 0)
        {
            _startPosX += _drawningSeaplaneWidth;
        }
        if (_drawningSeaplaneHeight - _startPosY < 0)
        {
            _startPosY += _drawningSeaplaneHeight;
        }
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntitySeaplane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntitySeaplane.Step > 0)
                {
                    _startPosX -= (int)EntitySeaplane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntitySeaplane.Step > 0)
                {
                    _startPosY -= (int)EntitySeaplane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                
                if (_startPosX + (int)EntitySeaplane.Step < _pictureWidth - 110)
                {
                    _startPosX += (int)EntitySeaplane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                
                if (_startPosY + (int)EntitySeaplane.Step < _pictureHeight - 60)
                {
                    _startPosY += (int)EntitySeaplane.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntitySeaplane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntitySeaplane.AdditionalColor);

        

        //границы автомобиля
        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 5, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 35, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 80, _startPosY.Value + 5, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 80, _startPosY.Value + 35, 20, 20);
        g.DrawRectangle(pen, _startPosX.Value + 9, _startPosY.Value + 15, 10, 30);
        g.DrawRectangle(pen, _startPosX.Value + 90, _startPosY.Value + 15, 10, 30);
        g.DrawRectangle(pen, _startPosX.Value + 20, _startPosY.Value + 4, 70, 52);

        //задние фары
        Brush brRed = new SolidBrush(Color.Red);
        g.FillEllipse(brRed, _startPosX.Value + 10, _startPosY.Value + 5, 20, 20);
        g.FillEllipse(brRed, _startPosX.Value + 10, _startPosY.Value + 35, 20, 20);

        //передние фары
        Brush brYellow = new SolidBrush(Color.Yellow);
        g.FillEllipse(brYellow, _startPosX.Value + 80, _startPosY.Value + 5, 20, 20);
        g.FillEllipse(brYellow, _startPosX.Value + 80, _startPosY.Value + 35, 20, 20);

        //кузов
        Brush br = new SolidBrush(EntitySeaplane.BodyColor);
        g.FillRectangle(br, _startPosX.Value + 10, _startPosY.Value + 15, 10, 30);
        g.FillRectangle(br, _startPosX.Value + 90, _startPosY.Value + 15, 10, 30);
        g.FillRectangle(br, _startPosX.Value + 20, _startPosY.Value + 5, 70, 50);

        //стекла
        Brush brBlue = new SolidBrush(Color.LightBlue);
        g.FillRectangle(brBlue, _startPosX.Value + 70, _startPosY.Value + 10, 5, 40);
        g.FillRectangle(brBlue, _startPosX.Value + 30, _startPosY.Value + 10, 5, 40);
        g.FillRectangle(brBlue, _startPosX.Value + 35, _startPosY.Value + 8, 35, 2);
        g.FillRectangle(brBlue, _startPosX.Value + 35, _startPosY.Value + 51, 35, 2);

        //выделяем рамкой крышу
        g.DrawRectangle(pen, _startPosX.Value + 35, _startPosY.Value + 10, 35, 40);
        g.DrawRectangle(pen, _startPosX.Value + 75, _startPosY.Value + 15, 25, 30);
        g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value + 15, 15, 30);

  
        

      
    }
}

